package cn.com.infosec.crypto.encodings;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.params.RSAKeyParameters;

public class ISO9796d1Encoding implements AsymmetricBlockCipher {
   private static byte[] shadows = new byte[]{14, 3, 5, 8, 9, 4, 2, 15, 0, 13, 11, 6, 7, 10, 12, 1};
   private static byte[] inverse = new byte[]{8, 15, 6, 1, 5, 2, 11, 12, 3, 4, 13, 10, 14, 9, 0, 7};
   private AsymmetricBlockCipher engine;
   private boolean forEncryption;
   private int bitSize;
   private int padBits = 0;

   public ISO9796d1Encoding(AsymmetricBlockCipher cipher) {
      this.engine = cipher;
   }

   private byte[] encodeBlock(byte[] in, int inOff, int inLen) {
      int var10000 = this.bitSize;
      int var4;
      byte[] var5 = new byte[var4 = (var10000 + 7) / 8];
      int var6 = this.padBits + 1;
      int var7 = (var10000 + 13) / 16;

      for(int var8 = 0; var8 < var7; var8 += inLen) {
         if (var8 > var7 - inLen) {
            int var9;
            int var10 = inOff + inLen - (var9 = var7 - var8);
            int var11 = var4 - var7;
            System.arraycopy(in, var10, var5, var11, var9);
         } else {
            System.arraycopy(in, inOff, var5, var4 - (var8 + inLen), inLen);
         }
      }

      int in1;
      byte inOff1;
      for(in1 = var4 - var7 * 2; in1 != var4; in1 += 2) {
         inOff1 = var5[var4 - var7 + in1 / 2];
         byte[] var14;
         var5[in1] = (byte)((var14 = shadows)[(inOff1 & 255) >>> 4] << 4 | var14[inOff1 & 15]);
         var5[in1 + 1] = inOff1;
      }

      var5[in1] = (byte)(var5[in1 = var4 - inLen * 2] ^ var6);
      var5[in1] = (byte)(var5[in1 = var4 - 1] << 4 | 6);
      var10000 = in1 = 8 - (this.bitSize - 1) % 8;
      inOff1 = 0;
      if (var10000 != 8) {
         var5[0] = (byte)(var5[0] & 255 >>> in1);
         var5[0] = (byte)(var5[0] | 128 >>> in1);
      } else {
         var5[0] = 0;
         var5[1] = (byte)(var5[1] | 128);
         inOff1 = 1;
      }

      return this.engine.processBlock(var5, inOff1, var4 - inOff1);
   }

   private byte[] decodeBlock(byte[] in, int inOff, int inLen) {
      byte[] var10000 = in = this.engine.processBlock(in, inOff, inLen);
      inOff = 1;
      inLen = (this.bitSize + 13) / 16;
      if ((var10000[var10000.length - 1] & 15) != 6) {
         throw new InvalidCipherTextException("invalid forcing byte in block");
      } else {
         int var4 = in.length - 1;
         in[var4] = (byte)((in[in.length - 1] & 255) >>> 4 | inverse[(in[in.length - 2] & 255) >> 4] << 4);
         byte[] var9;
         in[0] = (byte)((var9 = shadows)[(in[1] & 255) >>> 4] << 4 | var9[in[1] & 15]);
         boolean var10 = false;
         int var5 = 0;

         int var8;
         int var11;
         for(int var6 = in.length - 1; var6 >= in.length - inLen * 2; var5 = var8) {
            byte[] var7;
            var11 = (var7 = shadows)[(in[var6] & 255) >>> 4] << 4 | var7[in[var6] & 15];
            if (((in[var8 = var6 - 1] ^ var11) & 255) != 0) {
               if (var10) {
                  throw new InvalidCipherTextException("invalid tsums in block");
               }

               var10 = true;
               inOff = (in[var8] ^ var11) & 255;
            } else {
               var8 = var5;
            }

            var6 -= 2;
         }

         in[var5] = 0;
         byte[] var12 = new byte[var11 = (in.length - var5) / 2];

         for(inLen = 0; inLen < var11; ++inLen) {
            var12[inLen] = in[inLen * 2 + var5 + 1];
         }

         this.padBits = inOff - 1;
         return var12;
      }
   }

   public AsymmetricBlockCipher getUnderlyingCipher() {
      return this.engine;
   }

   public void init(boolean forEncryption, CipherParameters param) {
      RSAKeyParameters var3;
      if (param instanceof ParametersWithRandom) {
         var3 = (RSAKeyParameters)((ParametersWithRandom)param).getParameters();
      } else {
         var3 = (RSAKeyParameters)param;
      }

      this.engine.init(forEncryption, param);
      this.bitSize = var3.getModulus().bitLength();
      this.forEncryption = forEncryption;
   }

   public int getInputBlockSize() {
      ISO9796d1Encoding var10000 = this;
      int var1 = this.engine.getInputBlockSize();
      return var10000.forEncryption ? (var1 + 1) / 2 : var1;
   }

   public int getOutputBlockSize() {
      ISO9796d1Encoding var10000 = this;
      int var1 = this.engine.getOutputBlockSize();
      return var10000.forEncryption ? var1 : (var1 + 1) / 2;
   }

   public void setPadBits(int padBits) {
      if (padBits <= 7) {
         this.padBits = padBits;
      } else {
         throw new IllegalArgumentException("padBits > 7");
      }
   }

   public int getPadBits() {
      return this.padBits;
   }

   public byte[] processBlock(byte[] in, int inOff, int inLen) {
      return this.forEncryption ? this.encodeBlock(in, inOff, inLen) : this.decodeBlock(in, inOff, inLen);
   }
}
